package es.ucm.fdi.grasia.faerie.admin.gui;

import es.ucm.fdi.grasia.faerie.core.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Random;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import org.apache.felix.scr.annotations.Component;
import org.jgraph.graph.DefaultEdge;
import org.jgrapht.ListenableGraph;
import org.jgrapht.graph.ListenableDirectedGraph;
import org.apache.log4j.Logger;
import org.jgraph.JGraph;
import org.jgrapht.ext.JGraphModelAdapter;
import java.util.Iterator;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;

/**
 *
 * @author Chema
 */
@Component(inherit = true, factory = "es.ucm.fdi.grasia.faerie.adminGUI.ContextGraphViewerFactory")
@SuppressWarnings({ "unchecked", "rawtypes" })
public class ContextGraphViewer extends AbstractContextObserverComponent {

	private final Boolean graphLock = false;
	private ListenableGraph graph;
	private JGraphModelAdapter jgmodel;
	private JGraph jgraph;
	private JDialog contextGraphViewer;
	private Random rand = setUpRandom();

	protected Random setUpRandom() {
		return new Random(System.currentTimeMillis());
	}

	@Override
	public int handle(ContextEvent event) {
		return ContextObserver.HANDLE_READ;
	}

	@Override
	public void doActivate() {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (graphLock) {
					graph = new ListenableDirectedGraph<ContextElement, DefaultEdge>(DefaultEdge.class);
					jgmodel = new JGraphModelAdapter(graph);

					jgraph = new JGraph(jgmodel);
					jgraph.setConnectable(false);
					jgraph.setDisconnectable(false);
					jgraph.setEditable(false);

					contextGraphViewer =
							new JDialog((JFrame) null, ContextGraphViewer.this.getBundleEntity().toString());

					contextGraphViewer.add(new JScrollPane(jgraph));
					contextGraphViewer.setPreferredSize(new Dimension(800, 600));
					contextGraphViewer.pack();

					contextGraphViewer.setVisible(true);
				}
			}
		});
	}

	@Override
	public void elementAdded(ContextEvent event) {
		final ContextElement element = event.getContextElementArg();
		this.putObservedElement(element);
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (graphLock) {
					graph.addVertex(element);
					if (element instanceof Entity)
						setVertexColor(element, Color.BLUE);
					else if (element instanceof Attribute) {
						setVertexColor(element, Color.MAGENTA);
						Attribute att = (Attribute) element;
						graph.addEdge(att.getEntity(), att);
					} else if (element instanceof Relationship) {
						setVertexColor(element, Color.LIGHT_GRAY);
						Relationship rel = (Relationship) element;
						Entity entity = rel.getSourceEntity();
						if (!graph.containsVertex(entity))
							graph.addVertex(entity);
						graph.addEdge(rel.getSourceEntity(), rel);
						for (Iterator it = rel.getAllTargetEntities().iterator(); it.hasNext();) {
							Entity target = (Entity) it.next();
							if (!graph.containsVertex(target))
								graph.addVertex(target);
							graph.addEdge(rel, target);
						}
					}
					Rectangle b = adjustVertexBounds(element);
					positionVertexAt(element, rand.nextInt(jgraph.getWidth() - b.width), rand.nextInt(jgraph.getHeight()
							- b.height));
				}
			}
		});
	}

	@Override
	public void elementUpdated(ContextEvent event) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (graphLock) {
					jgraph.refresh();
				}
			}
		});
	}

	@Override
	public void elementRemoved(ContextEvent event) {
		Logger.getLogger(this.getClass()).trace("(" + event + ") ");
		final ContextElement element = event.getContextElementArg();
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (graphLock) {
					graph.removeVertex(element);
				}
			}
		});
		this.removeObservedElement(element);
	}

	@Override
	public void deactivate() {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (graphLock) {
					contextGraphViewer.dispose();
					graph = null;
					jgmodel = null;
					jgraph = null;
				}
			}
		});
		super.deactivate();
	}

	private void setVertexColor(Object vertex, Color color) {
		DefaultGraphCell cell = jgmodel.getVertexCell(vertex);
		AttributeMap attr = cell.getAttributes();

		GraphConstants.setBackground(attr, color);

		HashMap<DefaultGraphCell, AttributeMap> cellAttr = new HashMap<DefaultGraphCell, AttributeMap>();
		cellAttr.put(cell, attr);
		jgmodel.edit(cellAttr, null, null, null);
	}

	private Rectangle adjustVertexBounds(Object vertex) {
		DefaultGraphCell cell = jgmodel.getVertexCell(vertex);
		AttributeMap attr = cell.getAttributes();

		int length = vertex.toString().length();
		Rectangle b = GraphConstants.getBounds(attr).getBounds();
		Rectangle n = new Rectangle(b.x, b.y, length * 6 + 10, b.height);
		GraphConstants.setBounds(attr, n);

		HashMap<DefaultGraphCell, AttributeMap> cellAttr = new HashMap<DefaultGraphCell, AttributeMap>();
		cellAttr.put(cell, attr);
		jgmodel.edit(cellAttr, null, null, null);

		return n;
	}

	private void positionVertexAt(Object vertex, int x, int y) {
		DefaultGraphCell cell = jgmodel.getVertexCell(vertex);
		AttributeMap attr = cell.getAttributes();

		Rectangle b = GraphConstants.getBounds(attr).getBounds();
		GraphConstants.setBounds(attr, new Rectangle(x, y, b.width, b.height));

		HashMap<DefaultGraphCell, AttributeMap> cellAttr = new HashMap<DefaultGraphCell, AttributeMap>();
		cellAttr.put(cell, attr);
		jgmodel.edit(cellAttr, null, null, null);
	}
}
