package es.ucm.fdi.grasia.faerie.admin.gui;

import es.ucm.fdi.grasia.faerie.core.*;
import java.awt.Dimension;
import java.util.Enumeration;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import org.apache.felix.scr.annotations.Component;
import org.apache.log4j.Logger;

@Component(inherit = true, factory = "es.ucm.fdi.grasia.faerie.adminGUI.ContextViewerFactory")
@SuppressWarnings({ "rawtypes" })
public class ContextViewer extends AbstractContextObserverComponent {

	private final Boolean treeLock = false;
	private DefaultTreeModel tree;
	private JDialog contextViewer;

	@Override
	public int handle(ContextEvent event) {
		return ContextObserver.HANDLE_READ;
	}

	@Override
	public void activate() {
		super.activate();
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (treeLock) {
					tree = new DefaultTreeModel(new DefaultMutableTreeNode(), true);
					contextViewer = new JDialog((JFrame) null, ContextViewer.this.getBundleEntity().toString());
					contextViewer.add(new JScrollPane(new JTree(tree)));
					contextViewer.setPreferredSize(new Dimension(800, 600));
					contextViewer.pack();
					contextViewer.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
					contextViewer.setVisible(true);
					tree.reload();
				}
			}
		});
	}

	private DefaultMutableTreeNode findElementBreadth(ContextElement element) {
		synchronized (treeLock) {
			Enumeration e = ((DefaultMutableTreeNode) tree.getRoot()).breadthFirstEnumeration();
			while (e.hasMoreElements()) {
				DefaultMutableTreeNode result = (DefaultMutableTreeNode) e.nextElement();
				if (result.getUserObject() == element)
					return result;
			}
			return null;
		}
	}

	private DefaultMutableTreeNode findElementDepth(ContextElement element) {
		synchronized (treeLock) {
			Enumeration e = ((DefaultMutableTreeNode) tree.getRoot()).depthFirstEnumeration();
			while (e.hasMoreElements()) {
				DefaultMutableTreeNode result = (DefaultMutableTreeNode) e.nextElement();
				if (result.getUserObject() == element)
					return result;
			}
			return null;
		}
	}

	@Override
	public void elementAdded(ContextEvent event) {
		Logger.getLogger(this.getClass()).trace("(" + event + ")");
		final ContextElement element = event.getContextElementArg();
		this.putObservedElement(element);
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (treeLock) {
					if (element instanceof Entity) {
						Entity ent = (Entity) element;
						DefaultMutableTreeNode existing = findElementBreadth(ent);
						if (existing == null)
							tree.insertNodeInto(new DefaultMutableTreeNode(ent), (MutableTreeNode) tree.getRoot(), 0);
					} else if (element instanceof Attribute) {
						Attribute att = (Attribute) element;
						DefaultMutableTreeNode existing = findElementBreadth(att.getEntity());
						DefaultMutableTreeNode toInsert = new DefaultMutableTreeNode(att, false);
						if (existing != null)
							tree.insertNodeInto(toInsert, existing, 0);
						else {
							existing = new DefaultMutableTreeNode(att.getEntity());
							tree.insertNodeInto(existing, (MutableTreeNode) tree.getRoot(), 0);
							tree.insertNodeInto(toInsert, existing, 0);
						}
					}
					tree.reload();
				}
			}
		});
	}

	@Override
	public void elementUpdated(ContextEvent event) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (treeLock) {
					contextViewer.repaint();
				}
			}
		});
	}

	@Override
	public void elementRemoved(ContextEvent event) {
		Logger.getLogger(this.getClass()).trace("(" + event + ") ");
		final ContextElement element = event.getContextElementArg();
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (treeLock) {
					if (element instanceof Entity) {
						DefaultMutableTreeNode node = findElementBreadth(element);
						if (node != null)
							if (node.getChildCount() <= 0)
								tree.removeNodeFromParent(node);
							else
								node.setUserObject(null);
					} else if (element instanceof Attribute) {
						DefaultMutableTreeNode node = findElementDepth(element);
						if (node != null) {
							DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node.getParent();
							if (parent != null && parent.getChildCount() <= 1 && parent.getUserObject() == null)
								tree.removeNodeFromParent(parent);
							else
								tree.removeNodeFromParent(node);
						}
					}
					tree.reload();
				}
			}
		});
		this.removeObservedElement(element);
	}

	@Override
	public void deactivate() {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				synchronized (treeLock) {
					contextViewer.dispose();
					tree = null;
				}
			}
		});
		super.deactivate();
	}
}
